package be.iminds.ilabt.jfed.experimenter_gui.canvas;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.scene.Node;
import javafx.scene.shape.Line;

import javax.annotation.Nonnull;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * User: twalcari
 * Date: 11/8/13
 * Time: 12:19 PM
 */
public class CanvasLink extends Line {


    private static final String CANVAS_LINK_STYLECLASS = "canvas-link";
    private static final String CANVAS_LINK_SELECTED_STYLECLASS = "canvas-link";
    private static final String CANVAS_LINK_EDITABLE_STYLECLASS = "canvas-link-editable";

    private static final String CANVAS_LINK_TYPE_LAN_STYLECLASS = "canvas-link-type-lan";
    private static final String CANVAS_LINK_TYPE_GRE_STYLECLASS = "canvas-link-type-gre";
    private static final String CANVAS_LINK_TYPE_EGRE_STYLECLASS = "canvas-link-type-gre";
    private static final String CANVAS_LINK_TYPE_STITCH_STYLECLASS = "canvas-link-type-stitch";
    private static final String CANVAS_LINK_TYPE_WIRELESS_STYLECLASS = "canvas-link-type-wireless";
    private static final String CANVAS_LINK_TYPE_OTHER_STYLECLASS = "canvas-link-type-other";

    private final BooleanProperty selected = new SimpleBooleanProperty(false);
    private final BooleanProperty stitched = new SimpleBooleanProperty(false);
    private final StringProperty linkType = new SimpleStringProperty("lan");
    @Nonnull
    private final Node nodeA;
    @Nonnull
    private final Node nodeB;

    {
        /*
            Styling
         */

        //add listener to selected-property for styling
        selected.addListener((observableValue, oldValue, newValue) -> {
            if (newValue) {
                getStyleClass().remove(CANVAS_LINK_STYLECLASS);
                getStyleClass().add(CANVAS_LINK_SELECTED_STYLECLASS);

            } else {
                getStyleClass().remove(CANVAS_LINK_SELECTED_STYLECLASS);
                getStyleClass().add(CANVAS_LINK_STYLECLASS);

            }
        });

        getStyleClass().add(CANVAS_LINK_STYLECLASS);
        getStyleClass().add(CANVAS_LINK_EDITABLE_STYLECLASS);

        getStyleClass().add(CANVAS_LINK_TYPE_LAN_STYLECLASS);
        linkType.addListener((observableValue, oldValue, newValue) -> updateLinkTypeStyle());
        stitched.addListener((observableValue, oldValue, newValue) -> updateLinkTypeStyle());

        /*
            Event Listeners
         */
        // setOnMouseClicked(this);
    }

    public CanvasLink(@Nonnull Node nodeA, @Nonnull Node nodeB) {
        this.nodeA = nodeA;
        this.nodeB = nodeB;

        startXProperty().bind(NewLinkTarget.createCenterXBinding(nodeA));
        startYProperty().bind(NewLinkTarget.createCenterYBinding(nodeA));

        endXProperty().bind(NewLinkTarget.createCenterXBinding(nodeB));
        endYProperty().bind(NewLinkTarget.createCenterYBinding(nodeB));
    }

    private void updateLinkTypeStyle() {
        String linkTypeVal = linkType.getValue();

        Set<String> typesToremove = new HashSet<>(Arrays.asList(
                CANVAS_LINK_TYPE_LAN_STYLECLASS,
                CANVAS_LINK_TYPE_GRE_STYLECLASS,
                CANVAS_LINK_TYPE_EGRE_STYLECLASS,
                CANVAS_LINK_TYPE_STITCH_STYLECLASS,
                CANVAS_LINK_TYPE_WIRELESS_STYLECLASS,
                CANVAS_LINK_TYPE_OTHER_STYLECLASS));

        String newStyleClass;
        if (linkTypeVal == null) {
            newStyleClass = CANVAS_LINK_TYPE_LAN_STYLECLASS;
        } else {
            switch (linkTypeVal) {
                case "lan": { newStyleClass = CANVAS_LINK_TYPE_LAN_STYLECLASS; break; }
                case "gre-tunnel": { newStyleClass = CANVAS_LINK_TYPE_GRE_STYLECLASS; break; }
                case "egre-tunnel": { newStyleClass = CANVAS_LINK_TYPE_EGRE_STYLECLASS; break; }
                case "wireless": { newStyleClass = CANVAS_LINK_TYPE_WIRELESS_STYLECLASS; break; }

                //stitch does not really exists, but you never know...
                case "stitch": { newStyleClass = CANVAS_LINK_TYPE_STITCH_STYLECLASS; break; }

                default: { newStyleClass = CANVAS_LINK_TYPE_OTHER_STYLECLASS; break; }
            }
        }
        if (stitched.get())
            newStyleClass = CANVAS_LINK_TYPE_STITCH_STYLECLASS;
        typesToremove.remove(newStyleClass);

        getStyleClass().removeAll(typesToremove);
        getStyleClass().add(newStyleClass);
        List<String> currentStyleClasses = getStyleClass();
        System.out.println("stitched="+stitched.get());
        System.out.println("linkTypeVal="+linkTypeVal);
        System.out.println("typesToremove="+typesToremove);
        System.out.println("newStyleClass="+newStyleClass);
        System.out.println("currentStyleClasses="+currentStyleClasses);
    }

    @Nonnull
    public Node getNodeA() {
        return nodeA;
    }

    @Nonnull
    public Node getNodeB() {
        return nodeB;
    }


    public boolean isSelected() {
        return selected.get();
    }

    public void setSelected(boolean selected) {
        this.selected.set(selected);
    }

    public BooleanProperty selectedProperty() {
        return selected;
    }

    public boolean isStitched() {
        return stitched.get();
    }

    public BooleanProperty stitchedProperty() {
        return stitched;
    }

    public void setStitched(boolean stitched) {
        this.stitched.set(stitched);
    }

    public String getLinkType() {
        return linkType.get();
    }

    public StringProperty linkTypeProperty() {
        return linkType;
    }

    public void setLinkType(String linkType) {
        this.linkType.set(linkType);
    }
}
